/****************************************************************************
Date: 2/20/99
File: control.hpp
Description:	DOPPLER COMPENSATION CONTROLLER CLASS

	Coordinates and sequences interaction of satellite and radio objects.

	Public:
		CONTROLLER()
		VOID	SetVelocity(DOUBLE dVelocity)
		BOOL	SetConfig(const CHAR FAR* lpszSatName)
		VOID	Halt()
		BOOL	GetStatus()
		VOID	Process()
		VOID	Debug();
		~CONTROLLER()

History:

*****************************************************************************/
#ifndef CONTROL_HPP
#define CONTROL_HPP

#include "loctypes.h"
#include "sat.hpp"	// satellite base class
#include "radio.hpp"	// radio driver base class

class CONTROLLER {
public:
	// Sequencer ACTION function type
	typedef BOOL	(CONTROLLER::*CONTROLLERACTION)();

private:
	// Internal States
	enum CONTROLLER_STATE {		//***** CONTROLLER STATE **********
		CONTROLLER_STATE_STOP,	// not processing
		CONTROLLER_STATE_READY,	// ready for next velocity update
		CONTROLLER_STATE_RUN	// processing
	} eState;

	// Pointers to Satellite and Radio objects 
	SATELLITE*	pSat;
	RADIOSYSTEM*	pRx;
	RADIOSYSTEM*	pRx2;
	RADIOSYSTEM*	pTx;

	// Sequencer 
	VOID	Initialize();			// Initialize sequencer
	CONTROLLERACTION*	 pStep;		// pointer to next step
	static	CONTROLLERACTION aInitialize[];	// Initialize list
	static	CONTROLLERACTION aTune[];	// Tune radios list

	// Sequencer Action Functions
	BOOL	Reset();	// Reset sequencer back to top of Tune list

	BOOL	RxWait();	// Wait for receiver
	BOOL	Rx2Wait();	// Wait for 2nd receiver
	BOOL	TxWait();	// Wait for transmitter

	BOOL	RxInfoPoll();	// Poll receiver for parameters

	BOOL	RxTune();	// Tune receiver 
	BOOL	Rx2Tune();	// Tune 2nd receiver
	BOOL	TxTune();	// Tune transmitter

	BOOL	RxSetMode();	// Set rx modulation
	BOOL	Rx2SetMode();	// Set 2nd rx modulation
 	BOOL	TxSetMode();	// Set tx modulation
	BOOL	RxGetMode();	// Get current rx modulation type


	// Disallow Assignment and copy
	CONTROLLER(const CONTROLLER&);
	CONTROLLER& operator=(const CONTROLLER&);

public:
	CONTROLLER();
	VOID	SetVelocity(DOUBLE dVelocity);
	BOOL	SetConfig(const CHAR FAR* lpszSatName);
	VOID	Halt();
	BOOL	GetStatus() {
		return (BOOL) (eState != CONTROLLER_STATE_STOP);
	}
	VOID	Process();
	VOID	Debug();
	~CONTROLLER();
};

#endif // control.hpp
