/****************************************************************************
Date: 8/30/00
File: ft100.cpp
Description:
	FT100 radio driver

History:
	K Version	- turn off split

*****************************************************************************/
#include "ft100.hpp"


// Set Mode Table
// Index into table is MODULATION type, returns FT-100 mode code
const UCHAR FT100::aSetModeTable[] = {
	0x01,	// NA	(USB is default)
	0x00,	// LSB	
	0x01,	// USB
	0x02,	// CW
	0x06,	// FM
	0x04,	// AM
	0x05	// FSK	
};

// Get Mode Table
// index into table is lower 3 bits of FT-100 code, returns MODULATION type
const MODULATION FT100::aGetModeTable[] = {
		MODULATION_LSB,	// 0
		MODULATION_USB,	// 1
		MODULATION_CW,	// 2 
		MODULATION_CW,	// 3 (CW-R)
		MODULATION_AM,	// 4
		MODULATION_FSK,	// 5
		MODULATION_FM,	// 6
		MODULATION_FM,	// 7 FM Wide
};


/****************************************************************************
Function: SetFrequency()				PRIVATE
Description: Set radio to this frequency
Parameters:
	Steps = desired frequency in steps
Returns:
*****************************************************************************/
VOID	FT100::SetFrequency(LONG lFrequencySteps, LONG lStepSize) {

	// Convert Steps to frequency
	LONG	lFrequency = lFrequencySteps * lStepSize;

	//********************************************
	// Convert frequency to "Yaesu BCD" and
	// save in message params buffer
	// Note that frequency bytes are sent LSB first,
	// (Starting at Param1.)
	//********************************************
	LONG lDivisor	  = 100000000L; 	// (i.e. 100 MHz)
	LONG lTemp;
	for (SHORT nParam = FT100_MESSAGEP1; nParam >=0;  nParam--) {

		// Make upper nibble
		lTemp = lFrequency / lDivisor;
		lFrequency -= (lTemp * lDivisor);
		OutputBuffer.aParams[nParam] = (UCHAR) lTemp << 4;
		lDivisor /= 10;

		// Add in lower nibble
		lTemp = lFrequency / lDivisor;
		lFrequency -= (lTemp * lDivisor);
		OutputBuffer.aParams[nParam] += (UCHAR) lTemp;
		lDivisor /= 10;
	}

	SendMessage(FT100_SETFREQ);
	return;
}

/****************************************************************************
Function: FT100()
Description: constructor
Parameters:
	RADIOPARAMS structure
Returns:
*****************************************************************************/
FT100::FT100(const RADIOPARAMS& Params) :
	
	// Initialize
	RADIO(),
	Timer(),
	eState(STATE_IDLE),
	unMessageSize(0),
	ePTT(BOOL_FALSE),
	lRxVFOSteps(0),
	lRxVFORead(0),
	ucRxBand(0xff),
	eRxMode(MODULATION_NA),
	eUserHasTunedRx(BOOL_FALSE),
	lTxVFOSteps(0),
	eTxMode(MODULATION_NA),
	eTxLockout(BOOL_FALSE),
	Comport(),
	unByteCounter(0) {

	//********************************************************
	// Open com port and check communications with radio
	// If fails, return with Operational flag FALSE
	//********************************************************
	RADIO::eOperational = Comport.Open(
					Params.unPort,
					Params.unRate ?
						Params.unRate :
						FT100_BITRATE,
					FT100_DATABITS,
					FT100_PARITY,
					FT100_STOPBITS,
					Params.unIRQ,
					Params.Address);

	if (!RADIO::eOperational) {
		return;		// failed to open
	}

	//**********************************************************
	// Check communications with Radio.
	// Send test message to Radio and start a 1 second timer.
	// If timer expires before we receive test message response,
	// communications has failed.
	//***********************************************************

	// Some Laptop COM ports need a wake-up and radio seems to ignore
	// first message so send a harmless one and wait
	SendMessage(FT100_VFOMODE, FT100_VFOMODE_A);
	while(!Timer.IsExpired()) { }

	// Now see if radio is talking
	SendMessage(FT100_READMETER);
	USHORT		unCounter = 0;
	while(unCounter < FT100_READMETERSIZE) { 
		if (Timer.IsExpired()) {
 			// Radio failed to communicate
			RADIO::eOperational = BOOL_FALSE;
			return;
		}
		if (Comport.Getc() != COMDRV_EOF) {
			unCounter++;
		} 
	} // Radio is talking!

	// Always run in Split mode
	// A is receive, VFO B is TX
	// Select VFO A for Rx
	SendMessage(FT100_VFOMODE, FT100_VFOMODE_A);
	SendMessage(FT100_SPLITMODE, FT100_SPLITMODE_ON);

	return;
}

/****************************************************************************
Function: 	~FT100()
Description:	destructor
Parameters:
Returns:
*****************************************************************************/
FT100::~FT100() {

	SendMessage(FT100_SPLITMODE, FT100_SPLITMODE_OFF);
	while(!Timer.IsExpired()) { }	// wait 1 sec
	Comport.Close();
	return;
}

/****************************************************************************
Function: 	IsReady()
Description:
		Check if radio is ready for next command
Parameters:
Returns:
		BOOL_TRUE = Ready
		BOOL_FALSE = Not Ready
*****************************************************************************/

BOOL FT100::IsReady() { 
	if (eState == STATE_IDLE) {
		return BOOL_TRUE;
	}
	return BOOL_FALSE;
}

/****************************************************************************
Function: Update()
Description:
		Processes messages received from radio
Parameters:
	
Returns:
*****************************************************************************/
VOID	FT100::Update() {

	if (eState == STATE_IDLE) {
		return;			// do nothing
	}

	// Check for next message byte received
	INT nByte = Comport.Getc();
	if (nByte == COMDRV_EOF) {

		// Nope, Check for Timeout
		if (Timer.IsExpired()) {
			unByteCounter = 0;
			Comport.ClearBuffers();
			RefreshRxParams();
		}
		return;
	}

	// Got a byte, save it
	aInputBuffer[unByteCounter++] = (UCHAR) nByte;

	// if last byte in message, process it
	if ( unByteCounter == unMessageSize) {

		Timer.Stop();		// stop message timeout timer
		unByteCounter = 0;	// reset 

		switch(eState) {
		case STATE_READMETER:
			// Get PTT status
			if (aInputBuffer[0] == FT100_PTT) {
				ePTT = BOOL_TRUE;
				eTxLockout = BOOL_TRUE;
				eState = STATE_IDLE;
			}
			else {
				ePTT = BOOL_FALSE;
				// Not transmitting, send status update
				SendMessage(FT100_STATUSUPDATE);
				eState = STATE_STATUSUPDATE;
				unMessageSize = FT100_STATUSUPDATESIZE;
			}
			break;

		case STATE_STATUSUPDATE:

			// Check Rx band so we don't read in the
			// Tx band (workaround for radio CAT problem)

			// Is this the first update?
			if (ucRxBand == 0xff) {
				// Yes, set Rx band
				ucRxBand = aInputBuffer[0];
			}

			// Check read Rx band against saved Rx band
			if (ucRxBand != aInputBuffer[0]) {
				// No, OOPS! this is the Tx band

				// force freqs and modes to be
				// re-sent to radio
				lRxVFOSteps = 0;
				eRxMode = MODULATION_NA;

				lTxVFOSteps = 0;
				eTxMode = MODULATION_NA;

				// ignore this poll
				eState = STATE_IDLE;
			}
			else { 	// band is OK

				// Get Rx Mode
				ucRxModeRead = aInputBuffer[5];

				// Get Rx VFO Freq
				lRxVFORead = 0;
				for (INT i = 1; i <= 4; i++) {
					// Save Rx VFO
					lRxVFORead = lRxVFORead << 8;
					lRxVFORead |= aInputBuffer[i];
				}

				// Send "read meter" again to check PTT
				SendMessage(FT100_READMETER);
				eState = STATE_CHECK;
				unMessageSize = FT100_READMETERSIZE;
			}
			break;

		case STATE_CHECK:
			// Get PTT status
			if (aInputBuffer[0] == FT100_PTT) {
				ePTT = BOOL_TRUE;
				eTxLockout = BOOL_TRUE;
			}
			else {
				// Not transmitting, save Rx params
				ePTT = BOOL_FALSE;

				// Convert mode code to MODULATION type and save
				eRxMode = aGetModeTable[ucRxModeRead & 0x07];
				// Make into steps
				lRxVFORead = (lRxVFORead * 5) / 
						(4 * FT100_RXSTEPSIZE);

				// See if user has manually tuned radio
				// if so, save new freq steps
				if (lRxVFORead != lRxVFOSteps) {
					eUserHasTunedRx = BOOL_TRUE;
					lRxVFOSteps = lRxVFORead;

					eTxLockout = BOOL_TRUE;
				}
				else {
					// user has not tuned
					eTxLockout = BOOL_FALSE;
				}
			}
			eState = STATE_IDLE;
			break;
		default:
			// error case
			break;
		} // end switch()

	} // end of process received message

	return;
}

/****************************************************************************
Function: SetTxFreq()
Description:
	Set transmit frequency if different from current frequency
Parameters:
	Frequency
Returns:
*****************************************************************************/
VOID	FT100::SetTxFreq(DOUBLE dFrequency) {

	// If TX lockout is TRUE, ignore
	if (eTxLockout) {
		return;
	}

	// Calculate new frequency steps 
	LONG lNewTxVFOSteps = FreqtoSteps(dFrequency, FT100_TXSTEPSIZE);

	// Check to see if freq steps has changed
	// if so, update VFO steps and send new freq to radio
	if (lTxVFOSteps != lNewTxVFOSteps) {
		lTxVFOSteps = lNewTxVFOSteps;
		SendMessage(FT100_VFOMODE, FT100_VFOMODE_B);	// Tx VFO
		SetFrequency(lNewTxVFOSteps, FT100_TXSTEPSIZE);	// freq set
		SendMessage(FT100_VFOMODE, FT100_VFOMODE_A);	// back to Rx
	}
	return;
}

/****************************************************************************
Function: SetTxMode()
Description:
	Set transmitter modulation mode type
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	FT100::SetTxMode(MODULATION eMode) {

	if (eTxLockout) {
		return;
	}

	// Send message if different from current tx mode
	if (eTxMode != eMode) {
		eTxMode = eMode;	// Save new tx mode
		SendMessage(FT100_VFOMODE, FT100_VFOMODE_B);	// Tx VFO
		SetMode(eMode); 				// Tx mode
		SendMessage(FT100_VFOMODE, FT100_VFOMODE_A);	// back to Rx
	}
	return;
}

/****************************************************************************
Function: RefreshRxParams()
Description:
	Starts polling cycle
Parameters:
Returns:
*****************************************************************************/
VOID	FT100::RefreshRxParams() {

	SendMessage(FT100_VFOMODE, FT100_VFOMODE_A);
	SendMessage(FT100_READMETER);
	unMessageSize = FT100_READMETERSIZE;
	eState = STATE_READMETER;
	return;
}

/****************************************************************************
Function: GetRxFreq()
Description:
	Gets Receive (VFO) frequency
Parameters:
Returns:
	Frequency
*****************************************************************************/
DOUBLE	FT100::GetRxFreq() {

	eUserHasTunedRx = BOOL_FALSE;
	return (DOUBLE) (lRxVFOSteps * FT100_RXSTEPSIZE);
}

/****************************************************************************
Function: SetRxFreq()
Description:
	Set Receive frequency
Parameters:
	Frequency
Returns:
*****************************************************************************/
VOID	FT100::SetRxFreq(DOUBLE dFrequency) {

	eUserHasTunedRx = BOOL_FALSE;

	if (ePTT) {		// can't do it if transmitting
		return;
	}

	LONG lNewRxVFOSteps = FreqtoSteps(dFrequency, FT100_RXSTEPSIZE);
	if (lRxVFOSteps != lNewRxVFOSteps) {
		lRxVFOSteps = lNewRxVFOSteps;
		// Note the next line is to handle the case where the
		// user hits the PTT just as the SW is updating the
		// Tx freq and the radio gets out of sync.
		SendMessage(FT100_VFOMODE, FT100_VFOMODE_A);

		SetFrequency(lNewRxVFOSteps, FT100_RXSTEPSIZE);
	}

	return;
}

/****************************************************************************
Function: SetRxMode()
Description:
	Set receive modulation mode type.
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	FT100::SetRxMode(MODULATION eMode) {

	if (ePTT) {		// cannot do it if transmitting
		return;
	}

	// Change if new mode is different from current mode
	if (eRxMode != eMode) {
		SetMode(eMode);		// Send message to radio
		eRxMode = eMode; 	// Save new Rx mode
	}
	return;
}

// end FT-100.cpp
