/****************************************************************************
Date: 5/31/2000
File: icommsg.hpp	
Description:
	ICOM message handler class 

	This class handles low-level serial io and
	ICOM message encoding and decoding

     public:
	ICOMMSG();				// constructor
	BOOL Open(USHORT unICOMAddress,
		  UINT	unComPort,
		  UINT	unBitRate,
		  UINT	unIRQ,
		  IOADDRESS Address);

	VOID	Close();
	~ICOMMSG();				// destructor

	//***** Send messages to radio *****
	VOID	SendCmd(UCHAR ucCmd);
	VOID	SendCmd(UCHAR ucCmd, UCHAR ucSubCmd);
	VOID	SendWriteFrequency(LONG lFrequency);
	VOID	SendWriteMode(MODULATION eMode);

	//***** Get messages from radio *****
	BOOL	IsReady();
	UCHAR	GetCmd();
	LONG	GetFrequency();	// if GetCmd => ICOMMSG_READFREQ
	MODULATION GetMode();	// if GetCmd => ICOMMSG_READMODE


 	***** This is the ICOM Message Structure *****
	UCHAR	ucPreamble1;
	UCHAR	ucPreamble2;
	UCHAR	ucToAddress;
	UCHAR	ucFromAddress;
	UCHAR	ucCommand;
	UCHAR	aVariable[]; 
	UCHAR	EOM

History:
*****************************************************************************/
#ifndef ICOMMSG_HPP
#define ICOMMSG_HPP

#include "loctypes.h"	// local types
#include "comdrv.hpp"	// COM port driver class
#include "inttimer.hpp"	// interval timer
#include "modulate.hpp"	// MODULATION type

//***** ICOM Message Command and Sub Command codes *******************
const UCHAR ICOMMSG_READFREQ	= 0x03;	// read operating frequency
const UCHAR ICOMMSG_READMODE	= 0x04;	// read emission mode
const UCHAR ICOMMSG_WRITEFREQ	= 0x05;	// write operating frequency
const UCHAR ICOMMSG_WRITEMODE	= 0x06;	// write emission mode
const UCHAR ICOMMSG_VFOSELECT	= 0x07;	// Select VFO/MEMORY/BAND mode
const UCHAR ICOMMSG_SPLITMODE	= 0x0f;	// control split frequency mode
const UCHAR ICOMMSG_OK		= 0xfb;	// message ok (ACK)
const UCHAR ICOMMSG_NG		= 0xfa;	// message no good (NACK)

// for VFOSELECT command
const UCHAR ICOMMSG_VFOA	= 0x00;	// select vfo A
const UCHAR ICOMMSG_MAIN	= 0xd0;	// select MAIN band
const UCHAR ICOMMSG_SUB		= 0xd1; // select SUB Band

// for SPLITMODE command
const UCHAR ICOMMSG_CANCEL	= 0x00;	// cancel split mode
const UCHAR ICOMMSG_SELECT	= 0x01; // select split mode
const UCHAR ICOMMSG_DUPLEXMINUS	= 0x11;	// select full duplex (- tracking)
const UCHAR ICOMMSG_DUPLEXPLUS	= 0x12;	// select full duplex (+ tracking)
const UCHAR ICOMMSG_CANCELDUPLEX= 0x10; // turn duplex mode off


class ICOMMSG {
private:
	//***** Conversion tables between ICOM mode and MODULATION
	static const MODULATION	aGetModulation[];
	static const UCHAR	aGetICOMMode[];

	//***** Input and Output Buffers ******************************
	enum { MAXMSGSIZE = 12 };	// largest ICOM message (bytes)
	UCHAR		aInputBuffer[MAXMSGSIZE];
	USHORT		unInputBytes;
	UCHAR		aOutputBuffer[MAXMSGSIZE];
	USHORT		unOutputBytes;
	
	//***** Com port *********************************
	COMDRV		Comport; 	// Com port driver
	INTERVALTIMER	Timer;		// Com Timeout timer

	//*****	Utility Functions *************************
	VOID	SendMessage(UCHAR ucCmd, USHORT unDataCount);
	VOID	Resend();			// Resend last message

	// No copy constructor or assignment allowed
	// Force compile error if attempted
	ICOMMSG(const ICOMMSG&);
	ICOMMSG& operator=(const ICOMMSG&);
public:
	//***** General Functions ***************************
	ICOMMSG();				// constructor
	BOOL Open(USHORT unICOMAddress,
		  UINT	unComPort,
		  UINT	unBitRate,
		  UINT	unIRQ,
		  IOADDRESS Address);
	VOID	Close();
	~ICOMMSG();				// destructor

	//***** DEBUG ONLY! *****
	UCHAR*	pDebugInput;
	UCHAR*	pDebugOutput;

	//***** Send messages to radio **********************
	VOID	SendCmd(UCHAR ucCmd);
	VOID	SendCmd(UCHAR ucCmd, UCHAR ucSubCmd);
	VOID	SendWriteFrequency(LONG lFrequency);
	VOID	SendWriteMode(MODULATION eMode);

	//***** Get messages from radio *********************
	BOOL	IsReady();
	UCHAR	GetCmd();
	LONG	GetFrequency();	// if GetCmd => ICOMMSG_READFREQ
	MODULATION GetMode();	// if GetCmd => ICOMMSG_READMODE
};
#endif // icommsg.hpp
