/****************************************************************************
Date: 1/11/98
File: mbradio.cpp
Description:
	Mic Button Radio Driver

*****************************************************************************/
#include <math.h>
#include "mbradio.hpp"

// Constructor
MICBUTTONRADIO::MICBUTTONRADIO(const RADIOPARAMS& Params) :
	RADIO(),
	eState(STATE_BUSY) {

	if (Params.eTransceive) {	// no xcv operation possible so fail
		RADIO::eOperational = BOOL_FALSE;
		return;	
	}

	KbdTuner.Activate();

	// Setup initial frequency steps
	lStepSize = (LONG)
			(Params.ulFreqStep ? Params.ulFreqStep : DEFAULTSTEP);

	// Set the initial frequency steps
	lVFOFreqSteps = FreqtoSteps(Params.ulInitialFreq);
	lRITFreqSteps = 0;
	lActualFreqSteps = lVFOFreqSteps;

	eModulation = MODULATION_NA;

	RADIO::eOperational = PortDriver.Open(Params.unPort,Params.Address);

	return;
}

// Destructor
MICBUTTONRADIO::~MICBUTTONRADIO() {
	PortDriver.Close();
}

// Update() runs at DOS timer tick with interrupts off.
VOID MICBUTTONRADIO::Update() {

	// if UP or DOWN bits are set, clear them
	if (eState == STATE_BUSY) {
		PulseEnd();
		return;		// wait for next clock tick
	}
	
	// Update steps from keyboard tuner
	lVFOFreqSteps += KbdTuner.GetVFOSteps();
	lRITFreqSteps += KbdTuner.GetRITSteps();

	// Is Actual Frequency same as set frequency ?
	LONG lDiffSteps = lVFOFreqSteps + lRITFreqSteps - lActualFreqSteps;
	if (lDiffSteps > 0) {

		LEDOff();
		PulseUp();
		++lActualFreqSteps;
	}
	else {
		if (lDiffSteps < 0) {

			LEDOff();
			PulseDown();
			--lActualFreqSteps;
		}
		else {
			LEDOn();
		}
	}

	return;
}

// Tx Functions
VOID MICBUTTONRADIO::SetTxFreq(DOUBLE dFrequency) {
	volatile ULONG ulTempFreq = (ULONG) dFrequency;
	lVFOFreqSteps = FreqtoSteps(ulTempFreq);
}

// Rx Functions
BOOL	MICBUTTONRADIO::HasUserTunedRx() {
	return KbdTuner.HasUserTuned();
}

VOID	MICBUTTONRADIO::RefreshRxParams() {
	MODULATION eMode = KbdTuner.GetModulation();
	if (eMode != MODULATION_NA) {
		eModulation = eMode;
	}
}

DOUBLE MICBUTTONRADIO::GetRxFreq() {
	return (DOUBLE) (lVFOFreqSteps * lStepSize);
}

MODULATION MICBUTTONRADIO::GetRxMode() {
	return eModulation;
}

VOID MICBUTTONRADIO::SetRxFreq(DOUBLE dFrequency) {
	volatile ULONG ulTempFreq = (ULONG) dFrequency;
	lVFOFreqSteps = FreqtoSteps(ulTempFreq);
}

VOID	MICBUTTONRADIO::SetRxMode(MODULATION eMode) {
	eModulation = eMode;
}
// end mbradio.cpp
